package service;

import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import common.*;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * Created on : 14.01.2011
 * @author Sheroz Khaydarov
 */


@Controller
public class RestController {

	private ShopManager shopManager;

        public void setShopManager(ShopManager shopManager) {
            this.shopManager = shopManager;
        }
	
	@RequestMapping(method=RequestMethod.GET, value="/category", headers="Accept=application/xml")
	public @ResponseBody CategoryList getCategoryAll() {
		return new CategoryList(shopManager.getCategoryAll());
	}
	
	@RequestMapping(method=RequestMethod.GET, value="/category/{categoryId}", headers="Accept=application/xml")
	public @ResponseBody Category getCategory(@PathVariable String categoryId) {
		Category c = shopManager.getCategory(Long.parseLong(categoryId));
		return c;
	}

        @RequestMapping(method=RequestMethod.POST, value="/category")
	public @ResponseBody Category addCategory(@RequestBody Category c) {
		shopManager.addCategory(c);
		return c;
	}
	
	@RequestMapping(method=RequestMethod.PUT, value="/category/{categoryId}")
	public @ResponseBody Category updateCategory(@RequestBody Category c, @PathVariable String categoryId) {
		shopManager.updateCategory(c);
		return c;
	}
	
	@RequestMapping(method=RequestMethod.DELETE, value="/category/{categoryId}")
	public @ResponseBody void removeCategory(@PathVariable String categoryId) {
		shopManager.removeCategory(Long.parseLong(categoryId));
	}

        @RequestMapping(method=RequestMethod.GET, value="/category/{categoryId}/product", headers="Accept=application/xml")
	public @ResponseBody ProductList getProductAll(@PathVariable String categoryId, @RequestParam(value="sortOrder", required=false) String sort) {
            boolean sortOrderAsc=true;
            if (sort!=null && sort.equalsIgnoreCase("desc"))
                sortOrderAsc = false;

            return new ProductList(shopManager.getProductAll(Long.parseLong(categoryId), sortOrderAsc));
	}

	@RequestMapping(method=RequestMethod.GET, value="/category/{categoryId}/product/{productId}", headers="Accept=application/xml")
	public @ResponseBody Product getProduct(@PathVariable String categoryId, @PathVariable String productId) {
            Product product = shopManager.getProduct(Long.parseLong(categoryId), Long.parseLong(productId));
            return product;
	}

	@RequestMapping(method=RequestMethod.POST, value="/category/{categoryId}/product")
	public @ResponseBody Product addProduct(@PathVariable String categoryId, @RequestBody Product p) {
		shopManager.addProduct(Long.parseLong(categoryId), p);
		return p;
	}

	@RequestMapping(method=RequestMethod.PUT, value="/category/{categoryId}/product/{productId}")
	public @ResponseBody Product updateProduct(@PathVariable String categoryId, @RequestBody Product p, @PathVariable String productId) {
		shopManager.updateProduct(Long.parseLong(categoryId), p);
		return p;
	}

	@RequestMapping(method=RequestMethod.DELETE, value="/category/{categoryId}/product/{productId}")
	public @ResponseBody void removeCategory(@PathVariable String categoryId, @PathVariable String productId) {
		shopManager.removeProduct(Long.parseLong(categoryId), Long.parseLong(productId));
	}

}
