
package service;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import javax.sql.DataSource;
import org.apache.log4j.Logger;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import common.*;

/**
 * Created on : 14.01.2011
 * @author Sheroz Khaydarov
 */

public class ShopManager {
    private Logger log = Logger.getLogger(this.getClass());

    private List<Category> categoryList;
    private List<Product> productList;

    private JdbcTemplate jdbcTemplate;

    private static final class CategoryMapper implements RowMapper<Category> {
        public Category mapRow(ResultSet rs, int rowNum) throws SQLException {
            Category category = new Category();
            category.setId(rs.getLong("id"));
            category.setName(rs.getString("category_name"));
            return category;
        }
    }

    private static final class ProductMapper implements RowMapper<Product> {
        public Product mapRow(ResultSet rs, int rowNum) throws SQLException {
            Product product = new Product();
            product.setId(rs.getLong("id"));
            product.setCategoryId(rs.getLong("category_id"));
            product.setName(rs.getString("product_name"));
            product.setDescription(rs.getString("description"));
            product.setPrice(rs.getDouble("price"));
            return product;
        }
    }

    public void setDataSource(DataSource dataSource) {
        this.jdbcTemplate = new JdbcTemplate(dataSource);
    }

    public ShopManager() {

    }

    public List<Category> getCategoryAll()
    {
        categoryList = this.jdbcTemplate.query(
                "select * from navitest.category",
                new CategoryMapper());
        
        return categoryList;
    }

    public Category getCategory(long categoryId)
    {
        Category category;
        category = this.jdbcTemplate.queryForObject(
            "select * from navitest.category where id = ?",
            new Object[]{categoryId}, new CategoryMapper());

        return category;
    }

    public void addCategory(Category category)
    {
        // POST method
        if (category==null)
            return;

        this.jdbcTemplate.update("insert into navitest.category (category_name) values (?)", category.getName());
    }

    public void updateCategory(Category category)
    {
        // PUT method
        if (category==null)
            return;
        this.jdbcTemplate.update(
            "update navitest.category set category_name = ? where id = ?",
            category.getName(), category.getId());
    }

    public void removeCategory(long categoryId)
    {
        // DELETE method
        this.jdbcTemplate.update("delete from navitest.category where id = ?", categoryId);
    }

    public List<Product> getProductAll(long categoryId, boolean sortOrderAsc)
    {
        if (log.isDebugEnabled())
            log.debug("ShopManager.getProductAll(): categoryId = "+categoryId+", sortOrderAsc = "+sortOrderAsc);

        String query;
        query = "select * from navitest.product where category_id = ? order by price "+ (sortOrderAsc ? "asc" : "desc");

        productList = this.jdbcTemplate.query(query, new Object[]{categoryId}, new ProductMapper());
        return productList;
    }

    public Product getProduct(long categoryId, long productId)
    {
        if (log.isDebugEnabled())
            log.debug("ShopManager.getProduct(): categoryId:"+categoryId+", productId:"+productId);

        Product product;
        try {
            product = this.jdbcTemplate.queryForObject(
                "select * from navitest.product where id = ? and category_id = ?",
                new Object[]{productId, categoryId}, new ProductMapper());
        }
        catch (Exception e)
        {
            product=null;
        }

        if (product==null)
        {
            if (log.isDebugEnabled())
                log.debug("ShopManager.getProduct(): result is null");
        }
        
        return product;
    }

    public void addProduct(long categoryId, Product product)
    {
        // POST method
        if (product==null)
            return;

        this.jdbcTemplate.update("insert into navitest.product (category_id, product_name, description, price) values (?, ?, ?, ?)", categoryId, product.getName(), product.getDescription(), product.getPrice());
    }

    public void updateProduct(long categoryId, Product product)
    {
        // PUT method
        if (product==null)
            return;
        this.jdbcTemplate.update(
            "update navitest.product set category_id=?, product_name=?, description=?, price=? where id = ?",
            categoryId, product.getName(), product.getDescription(), product.getPrice(), product.getId());
    }
    
    public void removeProduct(long categoryId, long productId )
    {
        // DELETE method
        this.jdbcTemplate.update("delete from navitest.product where id = ? and category_id = ?", productId, categoryId);
    }

}
